#pragma once

#include "sicgl/blit.h"
#include "sicgl/color.h"
#include "sicgl/color_sequence.h"
#include "sicgl/debug.h"
#include "sicgl/domain/global.h"
#include "sicgl/domain/interface.h"
#include "sicgl/domain/screen.h"
#include "sicgl/extent.h"
#include "sicgl/field.h"
#include "sicgl/gamma.h"
#include "sicgl/interface.h"
#include "sicgl/iterator.h"
#include "sicgl/screen.h"
#include "sicgl/translate.h"
